@@ -141,6 +141,13 @@ class TestTextInputWidget(unittest.TestCase):
         pstruct = {}
         self.assertRaises(colander.Invalid, widget.deserialize, field, pstruct)
 
+    def test_deserialize_mask(self):
+        widget = self._makeOne(mask="999-99-9999")
+        field = DummyField()
+        pstruct = "123-45-6789"
+        result = widget.deserialize(field, pstruct)
+        self.assertEqual(result, "123-45-6789")
+
 
 class TestMoneyInputWidget(unittest.TestCase):
     def _makeOne(self, **kw):
@@ -1046,6 +1053,25 @@ class TestSelectWidget(unittest.TestCase):
         widget.serialize(field, None)
         self.assertEqual(renderer.kw["values"], [("1", "one")])
 
+    def test_serialize_generator_values(self):
+        renderer = DummyRenderer()
+        schema = DummySchema()
+        field = DummyField(schema, renderer)
+
+        def choices_generator():
+            for value in range(2):
+                yield (value, value)
+
+        choices = choices_generator()
+        widget = self._makeOne(values=choices)
+        with self.assertRaises(TypeError) as e:
+            widget.serialize(field, None)
+        self.assertTrue(type(e.exception) == TypeError)
+        self.assertEqual(
+            e.exception.args[0],
+            "Values must be a sequence type (list, tuple, or range).",
+        )
+
     def test_deserialize_null(self):
         widget = self._makeOne()
         field = DummyField()
@@ -1089,14 +1115,14 @@ class TestSelectWidget(unittest.TestCase):
     def test_selected_tag(self):
         """Test apply 'selected' to select."""
         widget = self._makeOne(multiple=False)
-        widget.get_select_value("foo", "foo") == "selected"
-        widget.get_select_value("foo", "bar") is None
+        assert widget.get_select_value("foo", "foo") == "selected"
+        assert widget.get_select_value("foo", "bar") is None
 
     def test_selected_tag_multiple(self):
         """Test apply 'selected' to multiselect."""
         widget = self._makeOne(multiple=True)
-        widget.get_select_value(["foo", "bar"], "foo") == "selected"
-        widget.get_select_value(["foo", "bar"], "moo") is None
+        assert widget.get_select_value(["foo", "bar"], "foo") == "selected"
+        assert widget.get_select_value(["foo", "bar"], "moo") is None
 
 
 class TestCheckboxChoiceWidget(unittest.TestCase):
@@ -1156,6 +1182,25 @@ class TestCheckboxChoiceWidget(unittest.TestCase):
         widget.serialize(field, None)
         self.assertEqual(renderer.kw["values"], [("1", "one")])
 
+    def test_serialize_generator_values(self):
+        renderer = DummyRenderer()
+        schema = DummySchema()
+        field = DummyField(schema, renderer)
+
+        def choices_generator():
+            for value in range(2):
+                yield (value, value)
+
+        choices = choices_generator()
+        widget = self._makeOne(values=choices)
+        with self.assertRaises(TypeError) as e:
+            widget.serialize(field, None)
+        self.assertTrue(type(e.exception) == TypeError)
+        self.assertEqual(
+            e.exception.args[0],
+            "Values must be a sequence type (list, tuple, or range).",
+        )
+
     def test_deserialize_null(self):
         widget = self._makeOne()
         field = DummyField()
@@ -1312,6 +1357,14 @@ class TestCheckedInputWidget(unittest.TestCase):
         pstruct = {"name": "x", "name-confirm": ["x"]}
         self.assertRaises(colander.Invalid, widget.deserialize, field, pstruct)
 
+    def test_deserialize_mask(self):
+        widget = self._makeOne(mask="999-99-9999")
+        field = DummyField()
+        pstruct = {"name": "123-45-6789", "name-confirm": "123-45-6789"}
+        result = widget.deserialize(field, pstruct)
+        self.assertEqual(result, "123-45-6789")
+        self.assertEqual(field.error, None)
+
 
 class TestCheckedPasswordWidget(TestCheckedInputWidget):
     def _makeOne(self, **kw):
