@@ -16,6 +16,7 @@ from iso8601.iso8601 import ISO8601_REGEX
 from translationstring import TranslationString
 
 from .compat import StringIO
+from .compat import sequence_types
 from .compat import string
 from .compat import string_types
 from .compat import text_
@@ -140,9 +141,21 @@ class Widget(object):
         be added to the input tag.
 
     requirements
-        A sequence of two-tuples in the form ``( (requirement_name,
-        version_id), ...)`` indicating the logical external
-        requirements needed to make this widget render properly within
+
+        The requirements are specified as a sequence of either of the
+        following.
+
+            1.  Two-tuples in the form ``(requirement_name, version_id)``.
+                The **logical** requirement name identifiers are resolved to
+                concrete files using the ``resource_registry``.
+            2.  Dicts in the form ``{requirement_type:
+                requirement_location(s)}``. The ``resource_registry`` is
+                bypassed.  This is useful for creating custom widgets with
+                their own resources.
+
+        Requirements specified as a sequence of two-tuples should be in the
+        form ``( (requirement_name, version_id), ...)`` indicating the logical
+        external requirements needed to make this widget render properly within
         a form.  The ``requirement_name`` is a string that *logically*
         (not concretely, it is not a filename) identifies *one or
         more* Javascript or CSS resources that must be included in the
@@ -152,7 +165,23 @@ class Widget(object):
         'tinymce' for Tiny MCE).  The ``version_id`` is a string
         indicating the version number (or ``None`` if no particular
         version is required).  For example, a rich text widget might
-        declare ``requirements = (('tinymce', '3.3.8'),)``.  See also:
+        declare ``requirements = (('tinymce', '3.3.8'),)``.
+
+        Requirements specified as a sequence of dicts should be in the form
+        ``({requirement_type: requirement_location(s)}, ...)``.  The
+        ``requirement_type`` key must be either ``js`` or ``css``.  The
+        ``requirement_location(s)`` value must be either a string or a list of
+        strings. Each string must resolve to a concrete resource. For example,
+        a widget might declare:
+
+        .. code-block:: python
+
+            requirements = (
+                {"js": "deform:static/tinymce/tinymce.min.js"},
+                {"css": "deform:static/tinymce/tinymce.min.css"},
+            )
+
+        See also:
         :ref:`specifying_widget_requirements` and
         :ref:`widget_requirements`.
 
@@ -282,7 +311,14 @@ class TextInputWidget(Widget):
     strip = True
     mask = None
     mask_placeholder = "_"
-    requirements = (("jquery.maskedinput", None),)
+    requirements = ()
+
+    def __init__(self, **kw):
+        super(TextInputWidget, self).__init__(**kw)
+        if getattr(self, "mask", False):
+            self.requirements = tuple(
+                list(self.requirements) + [("jquery.maskedinput", None)]
+            )
 
     def serialize(self, field, cstruct, **kw):
         if cstruct in (null, None):
@@ -842,7 +878,7 @@ class RichTextWidget(TextInputWidget):
     delayed_load = False
     strip = True
     template = "richtext"
-    requirements = (("tinymce", None),)
+    requirements = ({"js": "deform:static/tinymce/tinymce.min.js"},)
 
     #: Default options passed to TinyMCE. Customise by using :attr:`options`.
     default_options = (
@@ -1008,7 +1044,8 @@ class SelectWidget(Widget):
     **Attributes/Arguments**
 
     values
-        A sequence of items where each item must be either:
+        A sequence type (list, tuple, or range) of items where each item must
+        be either:
 
         - a two-tuple (the first value must be of type string, unicode
           or integer, the second value must be string or unicode)
@@ -1101,6 +1138,8 @@ class SelectWidget(Widget):
     multiple = False
     optgroup_class = OptGroup
     long_label_generator = None
+    selectize_options = None
+    default_selectize_options = (("allowEmptyOption", True),)
 
     def get_select_value(self, cstruct, value):
         """Choose whether <opt> is selected or not.
@@ -1123,8 +1162,18 @@ class SelectWidget(Widget):
             cstruct = self.null_value
         readonly = kw.get("readonly", self.readonly)
         values = kw.get("values", self.values)
+        if not isinstance(values, sequence_types):
+            e = "Values must be a sequence type (list, tuple, or range)."
+            raise TypeError(e)
         template = readonly and self.readonly_template or self.template
         kw["values"] = _normalize_choices(values)
+
+        selectize_options = dict(
+            kw.get("selectize_options")
+            or self.selectize_options
+            or self.default_selectize_options
+        )
+        kw["selectize_options_json"] = json.dumps(selectize_options)
         tmpl_values = self.get_template_values(field, cstruct, kw)
         return field.renderer(template, **tmpl_values)
 
@@ -1159,7 +1208,40 @@ class Select2Widget(SelectWidget):
     """
 
     template = "select2"
-    requirements = (("deform", None), ("select2", None))
+    requirements = (
+        ("deform", None),
+        {
+            "js": "deform:static/select2/select2.js",
+            "css": "deform:static/select2/select2.css",
+        },
+    )
+
+
+class SelectizeWidget(SelectWidget):
+    """
+    Renders ``<select>`` field based on a predefined set of values using
+    `selectize.js <https://github.com/selectize/selectize.js>`_ library.
+
+    **Attributes/Arguments**
+
+    Same as :func:`~deform.widget.SelectWidget`, with some extra options
+    listed here.
+
+    selectize_options: *dict* or *two-tuples*
+        Allows configuration of arbitrary options for Selectize. See
+        `Selectize docs under Usage for configuration options
+        <https://github.com/selectize/selectize.js/blob/master/docs/usage.md#configuration>`
+        for details.
+    """
+
+    template = "selectize"
+    requirements = (
+        ("deform", None),
+        {
+            "js": "deform:static/selectize/selectize.js",
+            "css": "deform:static/selectize/selectize.bootstrap3.css",
+        },
+    )
 
 
 class RadioChoiceWidget(SelectWidget):
@@ -1170,9 +1252,9 @@ class RadioChoiceWidget(SelectWidget):
     **Attributes/Arguments**
 
     values
-        A sequence of two-tuples (the first value must be of type
-        string, unicode or integer, the second value must be string or
-        unicode) indicating allowable, displayed values, e.g. ``(
+        A sequence type (list, tuple, or range) of two-tuples (the first value
+        must be of type string, unicode, or integer, the second value must be
+        string or unicode) indicating allowable, displayed values, e.g. ``(
         ('true', 'True'), ('false', 'False') )``.  The first element
         in the tuple is the value that should be returned when the
         form is posted.  The second is the display value.
@@ -1208,9 +1290,9 @@ class CheckboxChoiceWidget(Widget):
     **Attributes/Arguments**
 
     values
-        A sequence of two-tuples (the first value must be of type
-        string, unicode or integer, the second value must be string or
-        unicode) indicating allowable, displayed values, e.g. ``(
+        A sequence type (list, tuple, or range) of two-tuples (the first value
+        must be of type string, unicode or integer, the second value must be
+        string or unicode) indicating allowable, displayed values, e.g. ``(
         ('true', 'True'), ('false', 'False') )``.  The first element
         in the tuple is the value that should be returned when the
         form is posted.  The second is the display value.
@@ -1243,6 +1325,9 @@ class CheckboxChoiceWidget(Widget):
             cstruct = ()
         readonly = kw.get("readonly", self.readonly)
         values = kw.get("values", self.values)
+        if not isinstance(values, sequence_types):
+            e = "Values must be a sequence type (list, tuple, or range)."
+            raise TypeError(e)
         kw["values"] = _normalize_choices(values)
         template = readonly and self.readonly_template or self.template
         tmpl_values = self.get_template_values(field, cstruct, kw)
@@ -1314,7 +1399,14 @@ class CheckedInputWidget(Widget):
     confirm_subject = _("Confirm Value")
     mask = None
     mask_placeholder = "_"
-    requirements = (("jquery.maskedinput", None),)
+    requirements = ()
+
+    def __init__(self, **kw):
+        super(CheckedInputWidget, self).__init__(**kw)
+        if getattr(self, "mask", False):
+            self.requirements = tuple(
+                list(self.requirements) + [("jquery.maskedinput", None)]
+            )
 
     def serialize(self, field, cstruct, **kw):
         if cstruct in (null, None):
@@ -1376,7 +1468,7 @@ class CheckedPasswordWidget(CheckedInputWidget):
         retain and redisplay the provided values in the password inputs.  If
         false, on validation failure, the fields will be rendered empty.
         Default:: ``False``.
-        """
+    """
 
     template = "checked_password"
     readonly_template = "readonly/checked_password"
@@ -1554,7 +1646,10 @@ class SequenceWidget(Widget):
     min_len = None
     max_len = None
     orderable = False
-    requirements = (("deform", None), ("sortable", None))
+    requirements = (
+        ("deform", None),
+        {"js": "deform:static/scripts/jquery-sortable.js"},
+    )
 
     def prototype(self, field):
         # we clone the item field to bump the oid (for easier
@@ -1677,7 +1772,7 @@ class SequenceWidget(Widget):
 
 
 class filedict(dict):
-    """ Use a dict subclass to make it easy to detect file upload
+    """Use a dict subclass to make it easy to detect file upload
     dictionaries in application code before trying to write them to
     persistent objects."""
 
@@ -1714,7 +1809,7 @@ class FileUploadWidget(Widget):
     readonly_template = "readonly/file_upload"
     accept = None
 
-    requirements = (("fileupload", None),)
+    requirements = ({"js": "deform:static/scripts/file_upload.js"},)
 
     _pstruct_schema = SchemaNode(
         Mapping(),
@@ -2046,7 +2141,7 @@ class TextInputCSVWidget(Widget):
 
 
 class ResourceRegistry(object):
-    """ A resource registry maps :term:`widget requirement` name/version
+    """A resource registry maps :term:`widget requirement` name/version
     pairs to one or more relative resources.  A resource registry can
     be passed to a :class:`deform.Form` constructor; if a resource
     registry is *not* passed to the form constructor, a default
@@ -2067,21 +2162,21 @@ class ResourceRegistry(object):
             self.registry = {}
 
     def set_js_resources(self, requirement, version, *resources):
-        """ Set the Javascript resources for the requirement/version
+        """Set the Javascript resources for the requirement/version
         pair, using ``resources`` as the set of relative resource paths."""
         reqt = self.registry.setdefault(requirement, {})
         ver = reqt.setdefault(version, {})
         ver["js"] = resources
 
     def set_css_resources(self, requirement, version, *resources):
-        """ Set the CSS resources for the requirement/version
+        """Set the CSS resources for the requirement/version
         pair, using ``resources`` as the set of relative resource paths."""
         reqt = self.registry.setdefault(requirement, {})
         ver = reqt.setdefault(version, {})
         ver["css"] = resources
 
     def __call__(self, requirements):
-        """ Return a dictionary representing the resources required for a
+        """Return a dictionary representing the resources required for a
         particular set of requirements (as returned by
         :meth:`deform.Field.get_widget_requirements`).  The dictionary will be
         a mapping from resource type (``js`` and ``css`` are both keys in the
@@ -2132,8 +2227,6 @@ default_resources = {
             )
         }
     },
-    "tinymce": {None: {"js": "deform:static/tinymce/tinymce.min.js"}},
-    "sortable": {None: {"js": "deform:static/scripts/jquery-sortable.js"}},
     "typeahead": {
         None: {
             "js": "deform:static/scripts/typeahead.min.js",
@@ -2160,13 +2253,6 @@ default_resources = {
             ),
         }
     },
-    "select2": {
-        None: {
-            "js": "deform:static/select2/select2.js",
-            "css": "deform:static/select2/select2.css",
-        }
-    },
-    "fileupload": {None: {"js": "deform:static/scripts/file_upload.js"}},
 }
 
 default_resource_registry = ResourceRegistry()
